@@ -1,16 +1,17 @@
 //! Vectors can be really handy, sometimes. Do everything you want with your favorite direction pointing data type from mathematics.
 
-use super::general::{Increment, CommonPowers};
+use super::general::NumTools;
 const INV_DIM: &str = "Error: Dimensions did not match.";
 
 /// Mathematical Vectors in Rust.
+#[derive(Clone)]
 pub struct MathVector<T> {
     values: Vec<T>,
     length: Option<f64>,
 }
 
 impl<T: Copy +
-        super::general::CommonPowers +
+        NumTools<T> +
         From<f64> +
         std::ops::Mul<Output = T>> MathVector<T>
         where
@@ -111,7 +112,7 @@ impl<T: Copy +
 }
 
 impl<T: Copy +
-        super::general::CommonPowers +
+        super::general::NumTools<T> +
         From<f64> + Into<f64> +
         std::ops::Mul<Output = T> +
         std::ops::Add<Output = T>>
@@ -135,7 +136,7 @@ impl<T: Copy +
 }
 
 impl<T: Copy +
-        super::general::CommonPowers +
+        super::general::NumTools<T> +
         From<f64> +
         Into<f64> +
         std::ops::Mul<Output = T> +
@@ -169,7 +170,7 @@ impl<T: Copy +
 #[inline]
 #[must_use]
 pub fn scalar_mul<T: Copy +
-                     super::general::CommonPowers +
+                     super::general::NumTools<T> +
                      From<f64> +
                      std::ops::Mul<Output = T>>
                      (scalar: f64, other: &MathVector<T>) -> MathVector<T> 
@@ -205,7 +206,7 @@ macro_rules! new_mathvec {
 pub use new_mathvec;
 
 impl<T: Copy +
-        super::general::CommonPowers +
+        super::general::NumTools<T> +
         Into<f64> +
         std::fmt::Display + From<f64> +
         std::ops::Mul<Output = T>>
